/*
 * Copyright (c) 2018-2999 广州市蓝海创新科技有限公司 All rights reserved.
 *
 * https://www.mall4j.com/
 *
 * 未经允许，不可做商业用途！
 *
 * 版权所有，侵权必究！
 */

package com.qs.mp.web.controller.api.user;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.admin.domain.Coupon;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.TicketAwardsPrize;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.param.TicketBoxParam;
import com.qs.mp.admin.domain.param.TicketParam;
import com.qs.mp.admin.domain.vo.TicketBoxListVO;
import com.qs.mp.admin.domain.vo.TicketBoxVO;
import com.qs.mp.admin.domain.vo.TicketVO;
import com.qs.mp.admin.service.ITicketAwardsPrizeService;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.admin.service.ITicketService;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.ChannelAddr;
import com.qs.mp.channel.domain.ChannelCart;
import com.qs.mp.channel.domain.ChannelOrder;
import com.qs.mp.channel.domain.param.ChannelOrderParam;
import com.qs.mp.channel.domain.vo.ChannelCartVO;
import com.qs.mp.channel.domain.vo.ChannelOrderSettleVO;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.core.redis.RedisCache;
import com.qs.mp.common.enums.ChannelCertifyStatusEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.common.enums.TicketBoxStatusEnum;
import com.qs.mp.common.enums.TicketStatusEnum;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.framework.redis.RedisKey;
import com.qs.mp.user.domain.UserTicketOrder;
import com.qs.mp.user.domain.UserTicketOrderItem;
import com.qs.mp.user.domain.param.TicketOrderParam;
import com.qs.mp.user.domain.vo.TicketOrderSettleVO;
import com.qs.mp.user.domain.vo.UserCouponVO;
import com.qs.mp.user.service.IUserCouponService;
import com.qs.mp.user.service.IUserHitPrizeService;
import com.qs.mp.user.service.IUserTicketOrderItemService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import java.util.concurrent.TimeUnit;
import javax.validation.Valid;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


@RestController
@RequestMapping("/api/v1/mp/user/ticket")
@Api(tags = "用户端购买盲票接口")
@AllArgsConstructor
public class UserTicketOrderController extends BaseApiController {


  @Autowired
  private ITicketBoxService ticketBoxService;

  @Autowired
  private ITicketAwardsPrizeService ticketAwardsPrizeService;

  @Autowired
  private ITicketService ticketService;

  @Autowired
  private IUserTicketOrderItemService userTicketOrderItemService;

  @Autowired
  private IUserHitPrizeService userHitPrizeService;

  @Autowired
  private IUserCouponService userCouponService;

  @Autowired
  private MapperFacade mapperFacade;

  @Autowired
  private RedisCache redisCache;


  /**
   * 订单结算，每次更换优惠券时需重新请求此接口
   */
  @PostMapping("/order/settle")
  @ApiOperation(value = "订单结算" , notes = "购物车里点结算或者单个盲票组商品上点立即购买")
  public AjaxResult settle(@Valid @RequestBody TicketOrderParam param) {
    if (StringUtils.isBlank(param.getTicketId()) && StringUtils.isBlank(param.getBoxId())) {
      return AjaxResult.error("参数缺失");
    }
    Long userId = SecurityUtils.getLoginUser().getUserId();

    TicketOrderSettleVO orderSettleVO = new TicketOrderSettleVO();


    // 订单金额计算
    Integer orderAmt = 0;
    Integer discountAmt = 0;
    String picUrl;
    if (StringUtils.isNotBlank(param.getTicketId())) {
      // 线下扫码购票
      Ticket ticket = ticketService.getById(param.getTicketId());
      TicketBox ticketBox = ticketBoxService.getById(ticket.getBoxId());
      orderAmt = ticketBox.getSalePrice();
      picUrl = ticketBox.getPicUrl();
    } else {
      // 线上购票
      TicketBox ticketBox = ticketBoxService.getById(param.getBoxId());
      orderAmt = ticketBox.getSalePrice() * param.getOrderNum();
      picUrl = ticketBox.getPicUrl();
    }

    if (param.getAutoCoupon() == 1) {
      // 系统自动选择优惠券，按照金额从大到小排序
      List<UserCouponVO> userCouponList = userCouponService.listTicketOrderCoupon(userId);
      for (UserCouponVO userCouponVO : userCouponList) {
        if (orderAmt >= userCouponVO.getMinOrderAmt() && orderAmt >= userCouponVO.getDiscount()) {
          orderSettleVO.getCouponList().add(userCouponVO);
          discountAmt = userCouponVO.getDiscount();
        }
      }
    } else {
      // 查询指定券
    }
    orderSettleVO.setOrderAmt(orderAmt);
    orderSettleVO.setDiscountAmt(discountAmt);
    // 当前没有运费和优惠，实付金额=订单金额
    orderSettleVO.setPayAmt(orderAmt - discountAmt);
    orderSettleVO.setOrderNum(param.getOrderNum());
    orderSettleVO.setBoxId(param.getBoxId());
    orderSettleVO.setTicketId(param.getTicketId());
    orderSettleVO.setPicUrl(picUrl);

    // 缓存订单结算对象
    redisCache.setCacheObject(RedisKey.build(RedisKey.USER_TICKET_ORDER_KEY, userId), orderSettleVO, 10, TimeUnit.MINUTES);
    return AjaxResult.success(orderSettleVO);
  }


  /**
   * 提交订单
   */
  @PostMapping("/order/submit")
  @ApiOperation(value = "提交订单" , notes = "在订单确认页面提交")
  public AjaxResult submit(@RequestBody UserTicketOrder order) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    TicketOrderSettleVO orderSettleVO = redisCache.getCacheObject(RedisKey.build(RedisKey.USER_TICKET_ORDER_KEY, userId));
    if (null == orderSettleVO) {
      return AjaxResult.error("订单已过期，请重新下单");
    }
    //String orderId = channelOrderService.submitOrder(channelId, order.getMemo(), orderSettleVO);

    // 清除缓存的订单
    redisCache.deleteObject(RedisKey.build(RedisKey.USER_TICKET_ORDER_KEY, userId));

    return AjaxResult.success();
  }

}
